/*$Id: mg_in_old.cc,v 26.5 2006/11/05 07:19:10 al Exp $ -*- C++ -*-
 * Copyright (C) 2001 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
//testing=script 2006.10.31
#include "mg_.h"
/*--------------------------------------------------------------------------*/
static C_Comment   dummy_c_comment;
static Cxx_Comment dummy_cxx_comment;
/*--------------------------------------------------------------------------*/
void Parameter_1::parse(CS& file)
{
  file >> _type >> _code_name >> _comment;
  int here = file.cursor();
  for (;;) {
    ONE_OF 
      || get(file, "name",	     &_user_name)
      || get(file, "alt_name",	     &_alt_name)
      || get(file, "default",	     &_default_val)
      || get(file, "offset",	     &_offset)
      || set(file, "positive",	     &_positive,	true)
      || set(file, "octal",	     &_octal,		true)
      || get(file, "print_test",     &_print_test)
      || get(file, "calc_print_test",&_calc_print_test)
      || get(file, "scale",	     &_scale)
      || get(file, "calculate",	     &_calculate)
      || get(file, "quiet_min",	     &_quiet_min)
      || get(file, "quiet_max",	     &_quiet_max)
      || get(file, "final_default",  &_final_default)
      || get(file, "/*",	     &dummy_c_comment)
      || get(file, "//",	     &dummy_cxx_comment)
      ;
    if (file.skip1b(";,")) {
      break;
    }else if (!file.more()) {untested();
      file.warn(0, "premature EOF (parameter)");
      break;
    }else if (file.stuck(&here)) {untested();
      file.warn(0, "need ;");
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
void Code_Block::parse(CS& file) 
{
  // skips the code block, delimited by {}
  // sets _begin and _end, making that part of file the string
  // does not copy it.
  // also checks paren balance, so you can have {} inside the block

  int paren = file.skip1("{");
  if (paren == 0) {untested();
    file.warn(0, "need {");
  }else{
  }
  int here = file.cursor();
  _begin = file.tail();
  for (;;) {
    paren -= file.skip1b("])");
    if (paren == 0) {untested();
      file.warn(0, "unbalanced {}[]()");
      break;
    }else{
    }
    _end = file.tail();
    paren -= file.skip1b("}");
    if (paren == 0) {
      while (*(--_end) != '\n' && _end >= _begin) {
      }
      ++_end;
      break;
    }else{
    }
    if (file.stuck(&here)) {untested();
      file.warn(0, "syntax error");
      break;
    }else{
    }
    paren += file.skip1b("{[(");
    file.skip1b(";=");
    std::string foo;
    file >> foo;
  }
}
/*--------------------------------------------------------------------------*/
static void fill_in_default_names(Parameter_1_List& pl)
{
  for (Parameter_1_List::const_iterator
       p = pl.begin();
       p != pl.end();
       ++p) {
    (**p).fill_in_default_name();
  }
}
/*--------------------------------------------------------------------------*/
void Parameter_Block::parse(CS& file)
{ 
  int paren = file.skip1b("{");
  int here = file.cursor();
  for (;;) {
    ONE_OF
      || get(file, "unnamed",		&_unnamed_value)
      || get(file, "override",		&_override)
      || get(file, "raw_parameters",	&_raw)
      || get(file, "calculated_parameters",&_calculated)
      || get(file, "code_pre",		&_code_pre)
      || get(file, "code_mid",		&_code_mid)
      || get(file, "code_post",		&_code_post)
      || get(file, "/*",		&dummy_c_comment)
      || get(file, "//",		&dummy_cxx_comment)
      ;
    paren -= file.skip1b("}");
    if (paren == 0) {
      break;
    }else if (!file.more()) {untested();
      file.warn(0, "premature EOF (Parameter_Block)");
      break;
    }else if (file.stuck(&here)) {untested();
      file.warn(0, "bad Parameter_Code_Block");
      break;
    }else{
    }
  }
  fill_in_default_names(_calculated);
  fill_in_default_names(_raw);
  // but not _override
}
/*--------------------------------------------------------------------------*/
void Eval::parse(CS& file)
{
  file >> _name >> _code;
}
/*--------------------------------------------------------------------------*/
void Function::parse(CS& file)
{
  file >> _name;
  _name += '(' + file.ctos("{", "(", ")") + ')';
  file >> _code;
}
/*--------------------------------------------------------------------------*/
void Element_1::parse(CS& file)
{
  file >> _dev_type >> _name >> _port_list;
  int here = file.cursor();
  for (;;) {
    ONE_OF
      || get(file, "eval",	&_eval)
      || get(file, "value",	&_value)
      || get(file, "args",	&_args)
      || get(file, "omit",	&_omit)
      || get(file, "reverse",	&_reverse)
      || get(file, "state",	&_state)
      || get(file, "/*",	&dummy_c_comment)
      || get(file, "//",	&dummy_cxx_comment)
      ;
    if (file.skip1b(";")) {
      break;
    }else if (!file.more()) {untested();
      file.warn(0, "premature EOF (Element_1)");
      break;
    }else if (file.stuck(&here)) {untested();
      file.warn(0, "need ;");
      break;
    }else{
    }
  }
  _class_name = "DEV_" + to_upper(_dev_type);
}
/*--------------------------------------------------------------------------*/
void Arg::parse(CS& file)
{
  file.skipbl();
  _arg = file.get_to(";");
  file.skip1b(";");
}
/*--------------------------------------------------------------------------*/
void Circuit::parse(CS& file)
{
  int paren = file.skip1b("{");
  get(file, "/*", &dummy_c_comment);
  get(file, "//", &dummy_cxx_comment);
  get(file, "sync", &_sync) && file.skip1b(";");
  (get(file, "ports", &_required_nodes)
   && ((file >> _optional_nodes), file.skip1b(";")))
    || file.warn(0, "need ports");
  get(file, "local_nodes", &_local_nodes) && file.skip1b(";");
  int here = file.cursor();
  do {
    get(file, "args", &_args_list);
  } while (file.more() && !file.stuck(&here));
  for (;;) {
    paren -= file.skip1b("}");
    if (paren == 0) {
      break;
    }else if (!file.more()) {untested();
      file.warn(0, "premature EOF (Circuit)");
      break;
    }else{
    }
    ONE_OF
      || get(file, "/*", &dummy_c_comment)
      || get(file, "//", &dummy_cxx_comment)
      || (file >> _element_list)
      ;
    if (file.stuck(&here)) {untested();
      file.warn(0, "bad Circuit");
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
void Model::parse(CS& file)
{
  file >> _name;
  int paren = file.skip1b("{");
  int here = file.cursor();
  for (;;) {
    ONE_OF
      || get(file, "hide_base",		&_hide_base)
      || get(file, "level",		&_level)
      || get(file, "dev_type",		&_dev_type)
      || get(file, "inherit",		&_inherit)
      || get(file, "public_keys",	&_public_key_list)
      || get(file, "private_keys",	&_private_key_list)
      || get(file, "independent",	&_independent)
      || get(file, "size_dependent",	&_size_dependent)
      || get(file, "temperature_dependent", &_temperature)
      || get(file, "tr_eval",		&_tr_eval)
      || get(file, "validate",		&_validate)
      || get(file, "/*",		&dummy_c_comment)
      || get(file, "//",		&dummy_cxx_comment)
      ;
    paren -= file.skip1b("}");
    if (paren == 0) {
      break;
    }else if (!file.more()) {untested();
      file.warn(0, "premature EOF (Model)");
      break;
    }else if (file.stuck(&here)) {itested();
      file.warn(0, "bad Model");
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
void Device::parse(CS& file)
{
  file >> _name;
  int paren = file.skip1b("{");
  int here = file.cursor();
  for (;;) {
    ONE_OF
      || get(file, "parse_name",&_parse_name)
      || get(file, "id_letter",	&_id_letter)
      || get(file, "model_type",&_model_type)
      || get(file, "circuit",	&_circuit)
      || get(file, "tr_probe",	&_probes)
      || get(file, "device",	&_device)
      || get(file, "common", 	&_common)
      || get(file, "tr_eval",	&_tr_eval)
      || get(file, "eval",	&_eval_list)
      || get(file, "function",	&_function_list)
      || get(file, "/*",	&dummy_c_comment)
      || get(file, "//",	&dummy_cxx_comment)
      ;
    paren -= file.skip1b("}");
    if (paren == 0) {
      break;
    }else if (!file.more()) {untested();
      file.warn(0, "premature EOF (Device)");
      break;
    }else if (file.stuck(&here)) {untested();
      file.warn(0, "bad Device");
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
