/*$Id: mg_in_verilog.cc,v 26.3 2006/10/16 23:11:52 al Exp $ -*- C++ -*-
 * Copyright (C) 2001 Albert Davis
 * Author: Albert Davis <aldavis@ieee.org>
 *
 * This file is part of "Gnucap", the Gnu Circuit Analysis Package
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */
#include "mg_.h"
/*--------------------------------------------------------------------------*/
static C_Comment   dummy_c_comment;
static Cxx_Comment dummy_cxx_comment;
/*--------------------------------------------------------------------------*/
void Port_2::parse(CS& file)
{
  file >> _name;
  int here = file.cursor();
  for (;;) {
    ONE_OF
      || get(file, "short_to",	&_short_to)
      || get(file, "short_if",	&_short_if)
      ;
    if (file.skip1b(",")) {
      break;
    }else if (!file.more()) {
      file.warn(0, "premature EOF (Port_1)");
      break;
    }else if (file.stuck(&here)) {
      break;
    }
  }
}
/*--------------------------------------------------------------------------*/
void Element_2::parse(CS& file)
{
  file >> _dev_type >> '#' >> _parameter_list >> _name >> _port_list >> ';';
  _class_name = "DEV_" + to_upper(_dev_type);
}
/*--------------------------------------------------------------------------*/
void Parameter_2::parse(CS& file)
{
  file >> _code_name >> '=' >> _default_val;
  int here = file.cursor();
  for (;;) {
    ONE_OF 
      || get(file, "/*",	     &dummy_c_comment)
      || get(file, "//",	     &dummy_cxx_comment)
      ;
    if (!file.more()) {
      file.warn(0, "premature EOF (parameter)");
      break;
    }else if (file.stuck(&here)) {
      // skipcom is implied
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
void Parameter_3::parse(CS& file)
{
  file >> '.' >> _code_name >> '(' >> _default_val >> ')' >> ',';
}
/*--------------------------------------------------------------------------*/
void Attribute::parse(CS& file)
{
  file >> _name >> '=' >> _value >> ';';
}
/*--------------------------------------------------------------------------*/
/*
+ nature_declaration ::=
+     "nature" nature_name
+      [ nature_descriptions ]
+     "endnature"
+ nature_name ::=
+       nature_identifier
+     | nature_identifier : parent_identifier
+ parent_identifier ::=
+       nature_identifier
-     | discipline_identifier".flow"
-     | discipline_identifier".potential"
+ nature_descriptions ::=
+     nature_description { nature_description }
+ nature_description ::=
+     attribute "=" constant_expression ";"
+ attribute ::=
+       "abstol"
+     | "access"
+     | "ddt_nature"
+     | "idt_nature"
+     | "units"
-     | attribute_identifier
*/
void Nature::parse(CS& file)
{
  file >> _name;
  get (file, ":", &_parent_name);
 
  int here = file.cursor();
  for (;;) {
    ONE_OF
      || get(file, "/*",	&dummy_c_comment)
      || get(file, "//",	&dummy_cxx_comment)
      || get(file, "`",		&dummy_cxx_comment)
      || get(file, "abstol",	&_abstol)
      || get(file, "access",	&_access)
      || get(file, "ddt_nature",&_ddt_nature)
      || get(file, "idt_nature",&_idt_nature)
      || get(file, "units",	&_units)
      || (file >> _attributes)
      ;
    if (file.dmatch("endnature")) {
      break;
    }else if (!file.more()) {
      file.warn(0, "premature EOF (nature)");
      break;
    }else if (file.stuck(&here)) {
      file.warn(0, "bad nature");
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
/*
+ discipline_declaration ::=
+     "discipline" discipline_identifier
+      [ discipline_descriptions ]
+     "enddiscipline"
+ discipline_descriptions ::=
+     discipline_description { discipline_description }
+ discipline_description ::=
+       nature_binding
-     | attr_override
-     | domain_binding
+ nature_binding ::=
+     pot_or_flow nature_identifier ";"
- attr_override ::=
-     pot_or_flow "." attribute_identifier "=" constant_expression ";"
+ pot_or_flow ::=
+       "potential"
+     | "flow"
- domain_binding ::=
-       "domain discrete"
-     | "domain continuous"
*/
void Discipline::parse(CS& file)
{
  file >> _name;
 
  int here = file.cursor();
  for (;;) {
    ONE_OF
      || get(file, "/*",	&dummy_c_comment)
      || get(file, "//",	&dummy_cxx_comment)
      || get(file, "`",		&dummy_cxx_comment)
      || get(file, "potential",	&_potential)
      || get(file, "flow",	&_flow)
      ;
    if (file.dmatch("enddiscipline")) {
      break;
    }else if (!file.more()) {
      file.warn(0, "premature EOF (discipline)");
      break;
    }else if (file.stuck(&here)) {
      file.warn(0, "bad discipline");
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
/*
+ module_declaration ::=
+   {attribute_instance} module_keyword module_identifier 
+	[ digital_list_of_ports ] ";"
+   [ module_items ]
+   "endmodule"
+ module_keyword ::=
+     "module"
+   | "macromodule"
+ list_of_ports ::=
+     "(" port { "," port } ")"
+ port ::=
+       port_expression
-     | . port_identifier ( [ port_expression ] )
+ port_expression ::=
+       port_identifier
-     | port_identifier "[" constant_expression "]"
-     | port_identifier "[" constant_range "]"
- constant_range ::=
-     msb_constant_expression : lsb_constant_expression
+ module_items ::=
+     { module_item }
-   | analog_block
+ module_item ::=
?     module_item_declaration
-   | parameter_override
-   | module_instantiation
-   | digital_continuous_assignment
-   | digital_gate_instantiation
-   | digital_udp_instantiation
-   | digital_specify_block
-   | digital_initial_construct
-   | digital_always_construct
? module_item_declaration ::=
+     {attribute_instance} parameter_declaration
-   | {attribute_instance} local_parameter_declaration
-   | {attribute_instance} string_parameter_declaration
-   | {attribute_instance} local_string_parameter_declaration
-   | aliasparam_declaration
+   | {attribute_instance} digital_input_declaration
+   | {attribute_instance} digital_output_declaration
+   | {attribute_instance} digital_inout_declaration
-   | ground_declaration
-   | {attribute_instance} integer_declaration
-   | {attribute_instance} real_declaration
?   | {attribute_instance} net_discipline_declaration
-   | genvar_declaration
-   | branch_declaration
-   | analog_function_declaration
-   | digital_function_declaration
-   | digital_net_declaration
-   | digital_reg_declaration
-   | digital_time_declaration
-   | digital_realtime_declaration
-   | digital_event_declaration
-   | digital_task_declaration
- parameter_override ::=
-    "defparam" list_of_param_assignments ";"

+ input_declaration ::=
+    "input" [ range ] list_of_port_identifiers ";"
+ output_declaration ::=
+    "output" [ range ] list_of_port_identifiers ";"
+ inout_declaration ::=
+    "inout" [ range ] list_of_port_identifiers ";"

? net_discipline_declaration ::=
?     discipline_identifier [ range ] list_net_identifiers ";"
list_net_identifiers ::=
     net_identifier { , net_identifier }

*/
void Module::parse(CS& file)
{
  file >> _name >> _ports >> ';';
 
  int here = file.cursor();
  for (;;) {
    ONE_OF
      || get(file, "/*",	&dummy_c_comment)
      || get(file, "//",	&dummy_cxx_comment)
      || get(file, "`",		&dummy_cxx_comment)
      || get(file, "input",	&_input)
      || get(file, "output",	&_output)
      || get(file, "inout",	&_inout)
      || get(file, "electrical",&_electrical)
      || get(file, "parameter",	&_parameters)
      || (file >> _element_list)
      ;
    if (file.dmatch("endmodule")) {
      break;
    }else if (!file.more()) {
      file.warn(0, "premature EOF (module)");
      break;
    }else if (file.stuck(&here)) {
      file.warn(0, "bad module");
      break;
    }else{
    }
  }
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
